/*
    Fichier filtradr.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme dtruit les fichiers mail ne comportant pas
    d'adresse d'expditeur, ou provenant d'expditeurs refuss

    La liste des adresses interdites est mmorise dans un fichier
    qui peut contenir :
    - des adresses compltes
    - des adresses de fournisseurs d'accs
    - des adresses gnriques contenant des parties variables

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "trtentete.h"
#include "szchemin.h"


#define nbmax_adr     500     // nombre max d'adresses interdites
#define max_adr_gene   50     // nombre max d'adresses gnriques interdites


/* prototypes */
void charge_adr (FILE *fconf);
void testadr (int numes);
int  adr_generique (char *adresse);
int  adr_interdite (char *adresse);
int  adrgene_interdite (char *adresse);
int  test_adrgene (char *adresse, char *adrgene);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *listadr [nbmax_adr];   // mmorise les adresses interdites
char *listadrgene [max_adr_gene]; // mmorise les adresses gnriques interdites
int  sz_listadr;        // nombre d'adresses mmorises dans listadr
int  sz_listadrgene;    // nombre d'adresses mmorises dans listadrgene
FILE *ftrace;           // fichier contenant les expditeurs des mails refuss
char datecour [10];     // date courante (pour le fichier des mails refuss)
int  conserves = 0;     // nombre de mails conservs
int  supprimes = 0;     // nombre de mails supprims



/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;        // descripteur du fichier de configuration
    int  numes, nbmes;  // mumro du mail courant et nombre de mails
    char *ncom;         // nom de la commande que l'on excute (filtradr)


    // rcupration du nom de l'excutable
    ncom = varg [0];
    nbarg --;
    varg ++;

    // si le programme a t lanc avec l'option -t
    if (nbarg == 3 && strcmp (*varg, "-t") == 0)
    {
        // ouvrir le fichier trace en criture fin de fichier
        ftrace = fopen (varg [1], "a");

        // si nom de fichier correct
        if (ftrace)
            // initialiser la date courante
            initdatecour (datecour);
        else
            // sinon, avertir l'utilisateur
            fprintf (stderr, "Impossible d'crire dans le fichier %s\n",
                                                               varg [1]);

        // sauter les arguments traits
        nbarg -= 2;
        varg += 2;
    }
    // sinon, pas d'utilisation d'un fichier trace
    else
        ftrace = NULL;

    // controle du nombre d'arguments restants
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (*varg);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // charger en mmoire les adresses interdites
                charge_adr (fconf);

                // rcupration du nombre de mails
                nbmes = nbmails ();

                // rcupration de la liste des mails
                for (numes = 1; numes <= nbmes; numes++)
                    testadr (numes);

                // dition d'un rcapitulatif
                printf ("\n%d messages conservs, %d supprims\n",
                                          conserves, supprimes);

                // se dconnecter proprement du serveur pop
                deconnect_pop ();

                // si des mails ont t supprims
                if (supprimes)
                    // attendre pour enregistrement correct des suppressions
                    // (vite problme si autre filtre appel juste aprs)
                    sleep (2);
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
fprintf (stderr, "Syntaxe : %s [-t fichier_trace] fich_configuration\n", ncom);

    return (0);
}



/* charge en mmoire le fichier des adresses interdites */

void charge_adr (FILE *fconf)
{
    // fichier contenant les adresses refuses
    char ficadr_refus [szchemin + 10];
    char adresse [80]; // une adresse de ce fichier
    char *nouvadr;     // adresse retourne alloue dynamiquement
    FILE *frefus;      // descripteur du fichier des adresses
    int  i, j;         // compteurs


    // rcuprer le nom du rpertoire racine de la messagerie
    fgets (ficadr_refus, szchemin, fconf);

    // fabriquer le chemin d'accs au fichier contenant les adresses interdites
    strcpy (ficadr_refus + strlen (ficadr_refus) - 1, "/refus_adr");

    // initialisation : pas encore d'adresse mmorise
    sz_listadr     = 0;
    sz_listadrgene = 0;

    // accs au fichier des adresses refuses
    frefus = fopen (ficadr_refus, "r");

    // si ce fichier existe
    if (frefus)
    {
        // rcuprer et mmoriser les diffrentes adresses qu'il contient
        while (fgets (adresse, sizeof (adresse), frefus))
        {
            // allouer une variable pour mmoriser l'adresse lue
            nouvadr = (char *) malloc (strlen (adresse));

            if (! nouvadr)
            {
                fputs ("Problme d'allocation mmoire\n", stderr);
                exit (0);
            }

            // recopier d'adresse en la retournant
            // et en passant les caractres en minuscules
            i = strlen (adresse) - 2;
            j = 0;

            while (i >= 0)
                nouvadr [j++] = tolower (adresse [i--]);

            // terminer la chaine
            nouvadr [j] = 0;

            // mmoriser l'adresse dans la bonne liste ou message d'erreur
            switch (adr_generique (nouvadr))
            {
                          // adresse classique
                case  0 : if (sz_listadr < nbmax_adr)
                              listadr [sz_listadr ++] = nouvadr;
                          else
                          {
                               fprintf (stderr, "%s\n%s\n%s\n",
                "Trop d'adresses  filtrer, le filtrage ne sera que partiel",
                "Utiliser d'avantage les adresses gnriques",
                "ou recompiler l'application avec nbmax_adr plus grand");
                              return;
                          }

                          break;

                          // adresse gnrique valide
                case  1 : if (sz_listadrgene < max_adr_gene)
                              listadrgene [sz_listadrgene ++] = nouvadr;
                          else
                          {
                               fprintf (stderr, "%s\n%s\n%s\n%s\n",
                "Trop d'adresses gnriques  filtrer",
                "le filtrage ne sera que partiel",
                "Modifier les adresses gnriques",
                "ou recompiler l'application avec max_adr_gene plus grand");
                              return;
                          }

                          break;

                default : printf ("Adresse gnrique invalide : ");

                          // on affiche cette adresse dans le bon sens
                          while (--j >= 0)
                              putchar (nouvadr [j]);

                          putchar ('\n');

                          // et on peut librer le tableau nouvadr
                          free (nouvadr);
            }
        }

        // termin avec ce fichier
        fclose (frefus);
    }
    else
    {
        fprintf (stderr, "Pas de fichier %s\n%s\n", ficadr_refus,
      "seuls les mails avec une adresse d'expditeur sans @ seront supprims");
    }
}



/* lit l'entte d'un message, vrifie si l'adresse de l'expditeur
   est autorise, et supprime ce message si ce n'est pas le cas. */

void testadr (int numes)
{
    char bufw [80];     // buffer d'envoi d'une requte
    char bufFrom [80];  // buffer contenant le nom et l'adresse de l'expditeur
    char bufadr [80];   // buffer contenant l'adresse de l'expditeur retourne
    int  i, j;          // simples compteurs


    printf ("\rMessage %d", numes);
    fflush (stdout);

    // initialisation
    bufFrom [0] = '\0';
    bufadr  [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de l'adresse de l'expditeur
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si on a trouv le champ expditeur
        if (start ("From"))
        {
            // initialisations
            i = 5;
            j = 0;

            // recherche du @ dans l'adresse
            while (buf_lect [i] != '@' && buf_lect [i] != '\0')
                i++;

            // si l'adresse ne contient pas de @
            if (buf_lect [i] != '@')
            {
                // on va tester la ligne suivante
                lire_pop ();

                i = 1;

                while (buf_lect [i] != '@' && buf_lect [i] != '\0')
                    i++;
            }

            // une adresse valide devra contenir le @
            if (buf_lect [i] == '@')
            {
                // positionnement en fin d'adresse
                while (buf_lect [i] != '>' && buf_lect [i] != ' '
                                           && buf_lect [i] != '\0')
                    i++;

                i--;

                // recopie de l'adresse (dans l'ordre inverse des caractres)
                while (i > 0 && buf_lect [i] != '<' && buf_lect [i] != ' ')
                    bufadr [j++] = tolower (buf_lect [i--]);
            }

            // terminaison de la chaine
            bufadr [j] = 0;

            // si utilisation d'un fichier trace
            if (ftrace)
            {
                // convertir les caractres spciaux de la ligne
                majlignentete ();

                // initialisation
                i = 5;

                // recherche du dbut de l'expditeur
                while (buf_lect [i] == ' ')
                    i++;

                // tronquer si ncessaire l'expditeur (pour viter un plantage)
                if (strlen (buf_lect + i) > sizeof (bufFrom))
                     buf_lect [i + sizeof (bufFrom) - 1] = '\0';

                // mmorisation de l'expditeur
                strcpy (bufFrom, buf_lect + i);
            }
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // si le mail contient une adresse d'expditeur invalide ou interdite
    if (bufadr [0] == 0 || adr_interdite (bufadr) || adrgene_interdite (bufadr))
    {
        // demande de destruction du mail
        sprintf (bufw, "DELE %d", numes);
        env_pop (bufw);
        lire_pop ();
        supprimes ++;

        // si utilisation d'un fichier trace
        if (ftrace)
            // mmoriser l'adresse de l'expditeur de ce mail
            fprintf (ftrace, "%s%s\n", datecour, bufFrom);
    }
    else
        conserves ++;
}



/*  cherche si l'adresse passe en paramtre est une
    adresse classique ou une adresse gnrique valide */

int adr_generique (char *adresse)
{
    int retour = 0;


    // on teste tous les caractres de l'adresse
    // pour dtecter les mtas caractres des adresses gnriques
    while (*adresse)
    {
        // cas de 0, 1 ou plusieurs occurences d'un type de caractres
        if (*adresse == '+' || *adresse == '*')
        {
            // tester le mtacaractre suivant
            adresse ++;

            if (*adresse == 'a' || *adresse == '9' || *adresse == 'x')
                retour = 1;   // peut tre une adresse gnrique valide
            else
                return (-1);  // adresse gnrique invalide
        }

        // cas d'une suite de caractres variable de longueur fixe
        else if (*adresse == ']')
        {
            retour = 1;   // peut tre une adresse gnrique valide

            // analyse des mta caractres de l'intervalle
            do
                adresse++;
            while (*adresse == 'a' || *adresse == '9' || *adresse == 'x');

            // adresse gnrique invalide si caractre non autoris
            if (*adresse != '[')
                return (-1);
        }

        // passage au caractre suivant
        adresse ++;
    }

    // analyse de l'adresse termine
    return (retour);
}



/* recherche si l'adresse de l'expditeur est une adresse interdite */

int adr_interdite (char *adresse)
{
    int i, j;

    // on compare l'adresse expditeur avec toutes les adresses  refuser
    for (i = 0; i < sz_listadr; i++)
    {
        j = 0;

        // comparaison caractre par caractre
        while (adresse [j] == listadr [i][j])
        {
            if (adresse [j])
                j++;   // passage au caractre suivant
            else
                return (1); // trouv adresse refuse
        }

        if (listadr [i][j] == 0 && listadr [i][j-1] == '@')
            return (1);     // on refuse toutes les adresses de cet hbergeur
    }

    // l'adresse du mail ne fait pas partie des adresses interdites
    return (0);
}



/* recherche si l'adresse de l'expditeur correspond
    une adresses gnrique interdite */

int adrgene_interdite (char *adresse)
{
    int i;  // compteur


    // on compare l'adresse expditeur avec toutes les adresses  refuser
    for (i = 0; i < sz_listadrgene; i++)
    {
        if (test_adrgene (adresse, listadrgene [i]))
            return (1); // trouv adresse refuse
    }

    // l'adresse du mail ne correspond pas  une adresse gnrique interdite
    return (0);
}



/* comparaison entre l'adresse de l'expditeur et une adresse
   gnrique interdite (ou un morceau de ces 2 adresses) */

int test_adrgene (char *adresse, char *adrgene)
{
    // tant que l'adresse gnrique n'a pas t analyse entirement
    while (*adrgene)
    {
        // traitement distinct en fonction du
        // caractre courant de l'adresse gnrique
        switch (*adrgene)
        {
            // traitement suite de caractres variables de longueur fixe
            case ']' : adrgene++;

                       // cette suite est dlimite par des [ ]
                       while (*adrgene != '[')
                       {
                           // termin si fin prmature de l'adresse  comparer
                           if (! *adresse)
                               return 0;

                           switch (*adrgene)
                           {
                               // il faut trouver une lettre dans l'adresse
                               case 'a' : if (*adresse < 'a' || *adresse > 'z')
                                              return 0;
                                          break;

                               // il faut trouver un chiffer dans l'adresse
                               case '9' : if (*adresse < '0' || *adresse > '9')
                                              return 0;

                               // caractre quelconque accept dans l'adresse
                               default  : break;
                           }

                           // continuer l'analyse avec les caractres suivants
                           adresse ++;
                           adrgene ++;
                       }

                       // sauter le '[' de l'adresse gnrique
                       adrgene ++;
                       break;

            // traitement de 1 ou plusieurs caractres d'un type particulier
            case '+' : if (! *adresse)
                           return 0;   // termin si adresse trop courte
                       else
                       {
                           // test du caractre courant de l'adresse  comparer
                           switch (adrgene [1])
                           {
                               // cas o il faut que ce soit une lettre
                               case 'a' : if (*adresse < 'a' || *adresse > 'z')
                                              return 0;
                                          break;

                               // cas o il faut que ce soit un chiffre
                               case '9' : if (*adresse < '0' || *adresse > '9')
                                              return 0;

                               // cas gnral :n'importe quel caractre accept
                               default  : break;
                           }

                           // passer caractre suivant de l'adresse  comparer
                           adresse ++;
                       }

            // traitement de 0, 1 ou plusieurs caractres d'un type particulier
            case '*' : adrgene ++;  // rcuprer le type du caractre

                       // on va utiliser un appel rcursif pour
                       // faire une analyse dterministe
                       switch (*adrgene)
                       {
                           // prsence de 0, une ou plusieurs lettres avant
                           // la prochaine partie fixe de l'adresse
                           case 'a' : while ('a' <= *adresse && *adresse <= 'z')
                                      {
                                          if (test_adrgene (adresse, adrgene+1))
                                              return (1);

                                          adresse ++;
                                      }

                                      break;

                           // prsence de 0, un ou plusieurs chiffres avant
                           // la prochaine partie fixe de l'adresse
                           case '9' : while ('0' <= *adresse && *adresse <= '9')
                                      {
                                          if (test_adrgene (adresse, adrgene+1))
                                              return (1);

                                          adresse ++;
                                      }

                                      break;

                           // caractres quelconques avant la
                           // prochaine partie fixe de l'adresse
                           default  : while (*adresse)
                                      {
                                          if (test_adrgene (adresse, adrgene+1))
                                              return (1);

                                          adresse ++;
                                      }
                       }

                       // dernier essai, celui qui a le
                       // plus de chances de russir
                       return (test_adrgene (adresse, adrgene+1));
                       break;

            // cas gnral, comparaison de 2 caractres classiques
            default  : if (*adresse == *adrgene)   // si identiques
                       {
                           // continuer le test avec caractres suivants
                           adresse ++;
                           adrgene ++;
                       }
                       else
                           // sinon chec d'analyse
                           return (0);
        }
    }

    // succs si l'analyse des 2 adresses se termine simultanment
    return (*adresse == 0);
}
