/*
    Fichier filtresujet.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme dtruit les fichiers mail comportant une chaine de
    caractres particulire dans le sujet.

    La liste des chaines de caractres interdites est mmorise dans
    un fichier

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "testchamp.h"
#include "trtentete.h"
#include "szchemin.h"


#define nbmax_sujet   500     // nombre max de sujets interdits


/* prototypes */
int testsujet (int numes);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *listesujet [nbmax_sujet]; // mmorise les sujets interdits
int  sz_listesujet;      // nombre de sujets mmoriss dans listesujet
char bufSubject [80];    // buffer contenant le sujet du mail
int  opto, optO;         // options de traitement


/* programme principal */

int main (int nbarg, char *varg[])
{
    // fichier contenant la liste des chaines  tester
    char fichliste [szchemin + 12];
    FILE *fconf;         // descripteur du fichier de configuration
    FILE *ftrace;        // fichier contenant les sujets des mail refuss
    int  numes, nbmes;   // mumro du mail courant et nombre de mails
    char bufw [80];      // buffer d'envoi d'une requte de destruction de mail
    char datecour [10];  // date courante (pour le fichier des mails refuss)
    int  conserves = 0;  // nombre de mails conservs
    int  supprimes = 0;  // nombre de mails supprims
    char *ncom;          // nom de la commande que l'on excute (filtresujet)


    // rcupration du nom de l'excutable
    ncom = varg [0];
    nbarg --;
    varg ++;

    // initialisations valeur des options
    ftrace = NULL;
    opto = 0;
    optO = 0;

    // si le programme a t lanc avec des options
    while (nbarg >= 2 && **varg == '-')
    {
        // traitement des options
        switch (varg [0][1])
        {
            // option trace des mails supprims
            case 't' : // ouvrir le fichier trace en criture fin de fichier
                       ftrace = fopen (varg [1], "a");

                       // si nom de fichier correct
                       if (ftrace)
                           // initialiser la date courante
                           initdatecour (datecour);
                       else
                           // sinon, avertir l'utilisateur
                           fprintf (stderr,
                    "Impossible d'crire dans le fichier %s\n", varg [1]);

                       // sauter les arguments traits
                       nbarg -= 2;
                       varg  += 2;
                       break;

            // option test de la ligne d'entte d'origine (avant conversion)
            case 'o' : if (! optO)
                           opto = 1;
                       else
                           fputs ("L'option -O est prioritaire sur l'option -o\n",
                                                                        stderr);

                       // sauter l'argument trait
                       nbarg --;
                       varg  ++;
                       break;

            // option test de la ligne d'entte d'origine (avant conversion)
            case 'O' : optO = 1;

                       if (opto)
                           fputs ("L'option -O est prioritaire sur l'option -o\n",
                                                                        stderr);

                       // sauter l'argument trait
                       nbarg --;
                       varg  ++;
                       break;

            // aucune autre option n'est reconnue
            default  : fprintf (stderr, "Option %s incorrecte\n", *varg);
                       nbarg = 0;  // pour rappeller la syntaxe
        }
    }

    // controle du nombre d'arguments restants
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (*varg);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer le nom du rpertoire racine de la messagerie
                fgets (fichliste, szchemin, fconf);

                // fabriquer le chemin d'accs au fichier
                // contenant les sujets interdits
                strcpy (fichliste + strlen (fichliste) - 1, "/refus_sujet");

                // charger en mmoire les sujets interdits
                sz_listesujet = charge_valchamp (fichliste, listesujet,
                                                 nbmax_sujet, "nbmax_sujet");

                // si la liste n'est pas vide
                if (sz_listesujet)
                {
                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // vrification des diffrents mails
                    for (numes = 1; numes <= nbmes; numes++)
                    {
                        // si le sujet du mail contient une chaine interdite
                        if (testsujet (numes))
                        {
                            // demande de destruction du mail
                            sprintf (bufw, "DELE %d", numes);
                            env_pop (bufw);
                            lire_pop ();
                            supprimes ++;

                            // si utilisation d'un fichier trace
                            if (ftrace)
                                // mmoriser le sujet de ce mail
                                fprintf (ftrace, "%s%s\n", datecour, bufSubject);
                        }
                        else
                            conserves ++;
                    }

                    // dition d'un rcapitulatif
                    printf ("\n%d messages conservs, %d supprims\n",
                                              conserves, supprimes);
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();

                // si des mails ont t supprims
                if (supprimes)
                    // attendre pour enregistrement correct des suppressions
                    // (vite problme si autre filtre appel juste aprs)
                    sleep (2);
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        fprintf (stderr,
     "Syntaxe : %s [-(o|O)] [-t fichier_trace] fich_configuration\n", ncom);

    return (0);
}



/* lit l'entte d'un mail et vrifie si son sujet est autoris */

int testsujet (int numes)
{
    char bufw [80];      // buffer d'envoi d'une requte
    char sujetbrut [80]; // buffer contenant le sujet du mail avant dcodage
    int  debsujet;       // position du premier caractre significatif du sujet


    printf ("\rMessage %d", numes);
    fflush (stdout);

    // initialisations
    bufSubject [0] = '\0';
    sujetbrut  [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de le sujet du mail
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // si on a trouv le champ sujet
        if (start ("Subject"))
        {
            // recherche du premier caractre significatif du sujet
            debsujet = 8;

            while (buf_lect [debsujet] == ' ')
                debsujet++;

            // si option -o ou -O
            if (opto || optO)
            {
                // mmoriser le sujet du mail avant conversion en le tronquant
                // si ncessaire pour viter un dbordement de tableau
                if (strlen (buf_lect + debsujet) >= sizeof (sujetbrut))
                {
                    memcpy (sujetbrut, buf_lect + debsujet, sizeof (sujetbrut));
                    sujetbrut [sizeof (sujetbrut) - 1] = '\0';
                }
                else
                    strcpy (sujetbrut, buf_lect + debsujet);
            }

            // si on n'utilise pas l'option -O
            if (! optO)
            {
                // convertir les caractres spciaux de la ligne
                majlignentete ();

                // tronquer si ncessaire le sujet
                if (strlen (buf_lect + debsujet) >= sizeof (bufSubject))
                     buf_lect [debsujet + sizeof (bufSubject) - 1] = '\0';

                // mmorisation du sujet converti
                strcpy (bufSubject, buf_lect + debsujet);
            }
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // vrifier si le mail contient un des sujets de la liste interdite
    if (optO)
        // option -O : test sur le sujet brut seulement
        return trouve_valchamp (sujetbrut, listesujet, sz_listesujet);
    else if (opto)
    {
        // option -o (seule) : test sur le sujet brut et le sujet converti
        return (trouve_valchamp (sujetbrut,  listesujet, sz_listesujet)
            ||  trouve_valchamp (bufSubject, listesujet, sz_listesujet));
    }
    else
        // aucune option : test sur le sujet converti seulement
        return trouve_valchamp (bufSubject, listesujet, sz_listesujet);
}
