/*
    Fichier genentete.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Contient diffrentes fonctions pour :

    - insrer l'adresse de l'expditeur du mail qu'on cre
    - slectionner et insrer les destinataires
    - dater le mail
    - numroter les mails  envoyer
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "buflect.h"
#include "szchemin.h"
#include "genentete.h"
#include "modepage.h"
#include "carspe.h"


#define szbufadr      120   // longueur max d'une adresse dans le carnet
#define maxadr        500   // nombre max d'adresses dans le carnet
#define octet      unsigned char


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

static char *adresse [maxadr];   // les adresses de destinataires
static octet selection [maxadr]; // indique si le destinataire a t slectionn


/* sortie du programme sur erreur fatale */

void errfatale (char *format, char *argument)
{
    // crire le message d'erreur
    fprintf (stderr, format, argument);
    fputc ('\n', stderr);

    // pas indispensable, mais plus propre
    if (fmail)
        fclose (fmail);

    // sortir du programme
    exit (-1);
}



/* recupre le nom du rpertoire des mails  envoyer */

void recupdirenv (int nbarg, char *arg2)
{
    // rcuprer le nom du rpertoire des fichiers mail  envoyer
    if (getenv ("mailenv"))
        // rcupration dans variable environnement cre par vmaildir
        strcpy (direnv, getenv ("mailenv"));
    else
    {
        if (nbarg == 2)
            // rcupration dans 2me argument pass  la commande
            strcpy (direnv, arg2);
        else
        {
            // il ne reste plus que la saisie au clavier
            printf ("Nom du rpertoire racine des mails : ");
            fgets (direnv, szchemin, stdin);

            //  cause du \n retourn par fgets
            direnv [strlen (direnv) - 1] = '\0';
        }

        // si nom du rpertoire racine correct
        if (access (direnv, 0) == 0)
            // fabriquer le nom du rpertoire des mails  envoyer
            strcat (direnv, "/sortie");
        else
            // sinon, termin sur erreur
            errfatale ("Le rpertoire %s n'existe pas", direnv);
    }

    // crer le rpertoire d'envoi des mails si ncessaire
    mkdir (direnv, 0755);

    // s'il existe, on continue
    if (access (direnv, 0) < 0)
        errfatale ("Le rpertoire %s n'a pu tre cr", direnv);
}



/* rajoute un champ Message-Id  l'entte du mail */

void gen_mes_id (FILE *ficentete, long numail, char *adr_exped)
{
    int i;  // compteur


    // parcour de l'adresse de l'expditeur
    i = 0;

    // recherche du champ adresse
    while (adr_exped [i] && adr_exped [i] != '@')
        i++;

    // on se positionne au dbut de l'adresse
    while (i >= 0 && adr_exped [i] != '<' && adr_exped [i] != ' ')
        i--;

    // gnrer la ligne Message-Id
    fprintf (ficentete, "Message-Id: <%d-%s>\n", numail, adr_exped + i + 1);
}



/* rajoute la date et l'heure courante  la fin de l'entte du mail

   Le champ Date: gnr tient compte du rglage le plus frquent
   sur les ordinateurs europens : l'ordinateur est rgl sur l'heure
   locale et la valeur du fuseau horaire n'obit  aucune rgle stricte.
   Pour une utilisation dans des pays utilisant plusieurs fuseaux
   horaires, le rajout de celui ci pourra tre souhaitable.
 */

void ajout_date (char *nomfic)
{
    char commande [szchemin + 42];  // la commande que l'on va excuter

    // revenir  la langue de Bill Gates utilise pour les mails
    unsetenv ("LANG");     // les 3 premires variables concernent
    unsetenv ("LANGUAGE"); // les distributions Mandrake et Debian
    unsetenv ("LC_TIME");
    unsetenv ("LC_ALL");   // pour la distribution Red Hat

    // rajouter la date avec le bon format dans le fichier
    sprintf (commande, "date \"+%s\" >> %s", "Date: %a, %d %b %Y %T", nomfic);
    system (commande);
}



/* runit les 2 morceaux du mail et donne au fichier mail son nom dfinitif */

void fusionner (char *entete, long numail, char *message)
{
    FILE *fentet, *fmess;  // descripteurs des 2 fichiers
    int  car;              // caractre du corps du mail


    // ouvrir les 2 fichiers dans le bon mode
    fentet = fopen (entete, "a");

    if (! fentet)
        errfatale ("Impossible de complter l'entte du message %s", entete);

    fmess = fopen (message, "r");

    if (! fmess)
        errfatale ("Impossible de relire le corps du message %s", message);

    // saut de ligne obligatoire entre l'entte et le message
    putc ('\n', fentet);

    // copie des donnes (on rajoute le message  l'entte)
    car = getc (fmess);

    while (car != EOF)
    {
        putc (car, fentet);
        car = getc (fmess);
    }

    // fermer les fichiers
    fclose (fmess);
    fclose (fentet);

    // on peut dtruire le fichier temporaire qui contenait le message
    unlink (message);

    // donner au fichier mail son nom dfinitif
    sprintf (message, "%s/e%07d", direnv, numail);
    rename (entete, message);
}



/* ajoute la signature ventuelle  un mail */

void ajout_sign (char *message)
{
    char fic_sign [szchemin]; // fichier contenant la signature de l'expditeur
    FILE *fmess, *fsign;      // descripteur des fichiers message et signature
    int  car;                 // caractre de la signature


    // fabriquer le nom du fichier signature
    strcpy (fic_sign, direnv);
    strcpy (fic_sign + strlen (direnv) - 6, "signature");

    // essayer d'ouvrir ce fichier en lecture
    fsign = fopen (fic_sign, "r");

    // si le fichier signature a pu tre ouvert
    if (fsign)
    {
        // ouvrir le fichier contenant le mail en criture fin de fichier
        fmess = fopen (message, "a");

        // termin si problme  l'ouverture
        if (!fmess)
        {
            fputs ("Fichier mail protg en criture\n", stderr);
            sleep (2);
            return;
        }

        // rajout d'une sparation avannt la signature
        fputs ("--------------------------------\n", fmess);

        // copie de la signature
        car = getc (fsign);

        while (car != EOF)
        {
            putc (car, fmess);
            car = getc (fsign);
        }

        // fermer les fichiers
        fclose (fmess);
        fclose (fsign);
    }

    // sinon si le fichier signature existait quand mme
    else if (access (fic_sign, 0) == 0)
    {
        // message d'avertissement
        fprintf (stderr, "Fichier signature %s protg en lecture\n", fic_sign);
        sleep (2);
    }
}



/* charge en mmoire le contenu du carnet d'adresse */

void charge_carnet_adr ()
{
    char ficadr [szchemin];  // fichier contenant le carnet d'adresse
    FILE *fadr;              // descripteur du fichier carnet d'adresse
    char buf_adr [szbufadr]; // buffer pour lecture d'une adresse du carnet
    char *ligne_adr;         // ligne du carnet d'adresse  mmoriser
    int  i, taille;          // compteur + longueur ligne  mmoriser


    // initialisation : aucune adresse encore mmorise
    nb_lignes = 0;

    // trouver le nom du fichier carnet d'adresse et l'ouvrir en lecture
    strcpy (ficadr, direnv);
    strcpy (ficadr + strlen (direnv) - 6, "carnet-adr");
    fadr = fopen (ficadr, "r");

    // si ce carnet d'adresse existe
    if (fadr)
    {
        // mmoriser son contenu
        while (fgets (buf_adr, szbufadr, fadr))
        {
            taille = strlen (buf_adr) - 1;

            // supprimer les blancs ou parasites en fin de ligne
            while (taille > 0 && buf_adr [taille] <= ' ')
                taille --;

            // tester la validit de l'adresse
            for (i = 0; i < taille; i++)
                if (buf_adr [i] == '@')
                    break;

            // si adresse valide
            if (buf_adr [i] == '@')
            {
                // mmoriser si carnet d'adresse pas trop gros
                if (nb_lignes >= maxadr)
                    errfatale ("Carnet d'adresse trop grand.\n%s",
                        "Recompiler avec une valeur de maxadr suprieure");

                // allouer un buffer pour conserver la ligne
                ligne_adr = malloc (taille + 2);

                if (ligne_adr)
                {
                    // mmoriser l'entre du carnet d'adresse
                    memcpy (ligne_adr, buf_adr, taille + 1);
                    ligne_adr [taille + 1] = '\0';
                    adresse [nb_lignes ++] = ligne_adr;
                }
                else
                    errfatale ("Problme d'allocation mmoire", NULL);
            }
        }

        // termin avec le fichier carnet d'adresse
        fclose (fadr);
    }
}



/* rcupre le nom et l'adresse de l'expditeur */

void lire_exped ()
{
    char ficadr_exped [szchemin]; // fichier contenant l'adresse de l'expditeur
    FILE *fadr;                // descripteur du fichier adresse de l'expditeur


    // dtermine le nom du fichier contenant l'adresse de l'expditeur
    strcpy (ficadr_exped, direnv);
    strcpy (ficadr_exped + strlen (direnv) - 6, "adr_exped");

    // ouvrir ce fichier en lecture
    fadr = fopen (ficadr_exped, "r");

    // initialisation du buffer de lecture
    *buf_lect = '\0';

    // si l'ouverture s'est bien passe
    if (fadr)
    {
        // rcuprer l'information
        fgets (buf_lect, sz_buflect, fadr);
        fclose (fadr);
    }

    // si on n'a pas rcupr d'adresse
    if (! *buf_lect)
    {
        // demander  l'expditeur de saisir ces informations
        puts ("Nom, prnom et adresse Email de l'expditeur :");
        fgets (buf_lect, sz_buflect, stdin);

        // et les mmoriser dans un fichier pour les fois suivantes
        fadr = fopen (ficadr_exped, "w");

        if (fadr)
        {
            fputs (buf_lect, fadr);
            fclose (fadr);
        }
    }

    // supprimer le '\n' dans la chaine lue
    if (buf_lect [strlen (buf_lect) - 1] == '\n')
        buf_lect [strlen (buf_lect) - 1] = '\0';
}



/* slectionne des adresses de destinataires
   en tenant compte de celles dj utilises */

void choixdest (char *motcle)
{
    char buf_adr [szbufadr]; // buffer pour saisie adresse de destinataire
    int  car;                // caractre lu au clavier
    int  i, j, erreurs;      // compteurs


    // initialisation
    ajouts = 0;

    // si le carnet contient des adresses
    if (nb_lignes)
    {
        // aucun destinataire slectionn
        for (i = 0; i < nb_lignes; i++)
            selection [i] = 0;

        // initialisation affichage
        lignecour = 0;
        lignecran = 1;
        erreurs = 0;
        affpage ();

        do
        {
            // lire et traiter une touche du clavier
            car = leccar ();

            switch (car)
            {                  // dplacement dans la liste des destinataires
                case MONTE   : monte (1);
                               erreurs = 0;
                               break;

                case DESCEND : descend (1);
                               erreurs = 0;
                               break;

                case PAGEUP  : monte (lignepage + lignecran - 2);
                               erreurs = 0;
                               break;

                case PAGEDOWN: descend (2 * lignepage - lignecran - 2);
                               erreurs = 0;
                               break;

                case HOME    : monte (lignecour);
                               erreurs = 0;
                               break;

                case FIN     : descend (nb_lignes - lignecour - 1);
                               erreurs = 0;
                               break;

                               // slectionner une adresse
                case INSERT  : selection [lignecour] = 1;
                               affligne (lignecour);
                               break;

                               // dselectionner une adresse
                case SUPR    : selection [lignecour] = 0;
                               affligne (lignecour);
                               break;

                case ' '     : // slectionner ou dslectionner une adresse
                case '\n'    : selection [lignecour] = 1 - selection [lignecour];
                               affligne (lignecour);
                               break;

                case 0       : // aide si trop d'erreurs ou  la demande
                case F1      : effpage ();
                          puts ("Touches utilisables :\n");
                          puts ("flches Pageup, Pagedown, Home et Fin");
                          puts ("pour se dplacer d'une ou plusieurs lignes");
                          puts ("Inser pour rajouter le destinataire courant");
                          puts ("Suppr pour le dslectionner");
                          puts ("Espace ou Entre pour slect / dslect.");
                          puts ("Control L pour rafficher la page");
                          puts ("Esc pour valider la slection\n");
                          puts ("Appuyer sur une touche pour continuer");
                               leccar ();
                               // pas de break, on continue sur affpage

                case CTRL    : affpage ();
                               erreurs = 0;
                case ESC     : break;

                default      : putchar (7);  // bip

                               // si trop d'erreurs, afficher l'aide
                               if (++erreurs == 5)
                                   ungetc (0, stdin);  // on ira sur l'aide
            }
        }
        while (car != ESC);

        // On va recopier les adresses slectionnes
        // et les supprimer de la liste
        j = 0;

        for (i = 0; i < nb_lignes; i++)
        {
            // si adresse slectionne
            if (selection [i])
            {
                // ajoute ce destinataire dans l'entte du fichier mail
                ajout_adr (adresse [i], motcle);

                // on peut librer la mmoire occupe par l'adresse utilise
                free (adresse [i]);
            }
            // sinon, on la conserve
            else
                adresse [j++] = adresse [i];
        }

        // mise  jour du nombre d'adresses encore utilisables
        nb_lignes = j;

        // saisie manuelle d'autres descriptions d'utilisateurs
        effpage ();
        puts (
     "Vous pouvez prciser des destinataires inconnus dans le carnet d'adresse");
    }
    else
        puts (
        "Carnet d'adresse vide, vous pouvez choisir d'autres destinataires");

    puts ("Taper juste sur entre quand il n'y en a plus");

    // on repasse en mode saisie normale
    mode_normal ();

    do
    {
        printf ("Autre destinataire : ");
        i = 0;

        do
        {
            car = getchar ();

            // on ne mmorise pas le passage  la ligne
            if (car != '\n')
                buf_adr [i++] = car;
        }
        while (car != '\n');

        // on termine la chaine lue
        buf_adr [i] = '\0';

        // et si elle n'est pas vide, on mmorise le destinataire correspondant
        if (*buf_adr)
            ajout_adr (buf_adr, motcle);
    }
    while (*buf_adr);

    // configurer la liaison clavier pour lecture directe avec timeout
    mode_raw ();

    // terminaison de la liste de destinataires de la catgorie traite
    // on ne gnre pas de passage  la ligne si aucun
    // destinataire n'a t slectionn pour cette catgorie
    if (ajouts)
        fputc ('\n', fdest);
}



/* affiche une ligne de la liste des destinataires */

void affligne (int numlig)
{
    int i;    // simple compteur

    i = 0;

    putchar ('\r');

    if (selection [numlig])
        clair ();

    while (adresse [numlig][i] && i < colonpage)
        putchar (adresse [numlig][i++]);

    putchar ('\r');

    if (selection [numlig])
        lumnor ();
}



/* ajoute un destinataire dans l'entte du fichier mail
   on remet en forme les informations contenues dans la chaine destin
   pour les prsenter sous la forme : "description" <adresse>
   seule l'adresse est conserve por les destinataires en copie cache */

void ajout_adr (char *destin, char *motcle)
{
    int  debnom, finom, debadr, finadr; // position infos dans la chaine destin
    char buf_adr [szbufadr];  // buffer pour mise en forme de l'adresse
    int  szbuf;        // nombre de caractres mmoriss dans buf_adr
    static int occupe; // nombre de caractres dans ligne courante du fichier
    int  i;            // simple compteur (pour copie propre des donnes)

    // on va chercher le dbut et la fin de l'adresse mail
    finadr = strlen (destin) - 1;

    // recherche de la fin de l'adresse email
    while (finadr > 0 && (destin [finadr] == ' ' || destin [finadr] == '>'
                                                 || destin [finadr] == '\t'))
        finadr --;

    // recherche du dbut de l'adresse email
    debadr = finadr - 1;

    while (debadr >= 0 && destin [debadr] != ' ' && destin [debadr] != '<'
                                                 && destin [debadr] != '\t')
        debadr --;

    debadr ++;
    szbuf = 0;

    // si l'on ne s'occupe pas des destinataires en copie cache
    if (*motcle != 'B')
    {
        // recherche de la fin du nom du destinataire
        finom = debadr - 2;

        while (finom > 0 && (destin [finom] == ' ' || destin [finom] == '"'
                                                   || destin [finom] == '\t'))
            finom --;

        // recherche du dbut du nom du destinataire
        debnom = 0;

        while (destin [debnom] == ' ' || destin [debnom] == '"'
                                      || destin [debnom] == '\t')
            debnom ++;

        // si la description du destinataire est prsente
        if (debnom < finom)
        {
            // la copier
            buf_adr [szbuf++] = '"';

            // l'utilisation de la variable i permet de ne pas
            // modifier debnom qui pourtant ne resservira plus
            for (i = debnom; i <= finom; i++)
                buf_adr [szbuf++] = destin [i];

            buf_adr [szbuf++] = '"';
            buf_adr [szbuf++] = ' ';
        }
    }

    // copie de l'adresse email
    buf_adr [szbuf++] = '<';

    // en caractres minuscules
    for (i = debadr; i <= finadr; i++)
        buf_adr [szbuf++] = tolower (destin [i]);

    buf_adr [szbuf++] = '>';
    buf_adr [szbuf++] = '\0';

    // tape 2 : on copie les informations sur le destinataire dans le fichier

    // si c'est le premier destinataire de cette catgorie
    if (! ajouts)
    {
        // on commence la liste
        fprintf (fdest, "%s:", motcle);

        // initialisations
        occupe = strlen (motcle);
        ajouts = 1;
    }
    // sinon, mettre une virgule aprs le destinataire prcdent
    else
        fputc (',', fdest);

    // passage  la ligne ?
    if (occupe + strlen (buf_adr) >= 80 && occupe > 4)
    {
        fprintf (fdest, "\n   ");
        occupe = 3;
    }

    // copie de la description du destinataire
    fprintf (fdest, " %s", buf_adr);

    // mise  jour du compteur d'octets dans la ligne
    occupe = occupe + strlen (buf_adr) + 2;
}
