/*
    Fichier sjfmails.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme affiche les caractristique de chaque mail
    mmoris dans un fichier.

    Il y a 2 formats d'affichage.

    Le format dtaill (option -d) donne plusieurs lignes d'information :
    - date et heure d'envoi
    - expditeur et destinataire(s) + ventuelle adresse de rponse
    - sujet du mail
    - format du message (texte, html ...)
    - ventuellement premires lignes du message

    Le format simplifi (option -s) affiche une ligne par message :
      expditeur     sujet     date et heure d'envoi
     destinataire    sujet     date et heure d'envoi
    selon qu'il s'agisse d'un mail reu ou envoy (ou  envoyer)

    Par dfaut, on analyse les mails du rpertoire courant.
    On peut aussi indiquer un autre rpertoire de recherche.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"
#include "trtligne.h"


#define nbmaxfic       2000
#define sz_nom           20
#define sz_avantdate     67


/* prototypes */
void lister_mails ();
void trinoms ();
void aff_detail (char *nomfic);
void aff_resume (char *nomfic);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

int  nblig_mes;               // nombre de lignes  afficher pour chaque message
char fichmail [nbmaxfic][12]; // noms des fichiers mail
int  nbmails;                 // nombre de fichiers mail


/* programme principal */

int main (int nbarg, char *varg[])
{
    int  opt_aff;  // option affichage (simple = -s, dtaill = -d)
    int  numail;
    char *dircom, *dirmails;


    // recupration nom de la commande (pour message d'erreur)
    dircom = *varg;

    // initialisation options de fonctionnement
    opt_aff = 0;
    nblig_mes = 0;

    // rcupration des options de fonctionnement
    if (--nbarg > 0)
    {
        varg ++;

        if (strcmp (*varg, "-s") == 0)
        {
            varg ++;
            nbarg --;
        }
        else if (strcmp (*varg, "-d") == 0)
        {
            opt_aff = 1;
            varg ++;
            nbarg --;

            if (nbarg > 0)
            {
                nblig_mes = atoi (*varg);

                if (nblig_mes)
                {
                    varg ++;
                    nbarg --;
                }
            }
        }
        else if (**varg == '-')
        {
            fprintf (stderr, "Option %s inexistante\n", *varg);
            nbarg = 2;   // pour rappeller la syntaxe de la commande
        }
    }

    // rcupration ventuelle du nom du rpertoire des mails
    if (nbarg == 1)
    {
        dirmails = *varg;

        // positionnement dans ce rpertoire
        if (chdir (dirmails) < 0)
        {
            fprintf (stderr, "Rpertoire %s inexistant\n", dirmails);
            return (0);
        }
    }

    // si aucune erreur dans la liste des paramtres
    if (nbarg < 2)
    {
        // rcupration de la liste des fichiers mail
        lister_mails ();

        // ordonns par noms de fichiers
        trinoms ();

        // affichage dtaill ou rsum de l'entte des mails
        if (opt_aff)
            for (numail = 0; numail < nbmails; numail++)
                aff_detail (fichmail [numail]);
        else
            for (numail = 0; numail < nbmails; numail++)
                aff_resume (fichmail [numail]);
    }
    else
        fprintf (stderr,
         "Syntaxe : %s [-(s|d [nb_lignes_messages])] [rpertoire_fichiers]\n",
                 dircom);

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* rcupre le nombre de mails */

void lister_mails ()
{
    DIR *repert;
    struct dirent *entree;


    // initialisation
    nbmails = 0;

    // accder au rpertoire
    repert = opendir (".");

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire
    while (entree)
    {
        // mmoriser le nom aprs controle
        if (strlen (entree->d_name) == 8 ||
           (strlen (entree->d_name) == 10 && entree->d_name [8] == '.'))
            strcpy (fichmail [nbmails++], entree->d_name);

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }
}



/* tri alphabtique des noms de fichiers mail */

void trinoms ()
{
    char tampon [12]; // chaine de caractres pour permutation
    int  i;           // indice dans la liste


    // initialisation
    i = 0;
    while (i < nbmails - 1)
    {
        // on compare 2 lments conscutifs
        if (strcmp (fichmail [i], fichmail [i+1]) > 0)
        {
            // interversion de 2 noms
            strcpy (tampon, fichmail [i]);
            strcpy (fichmail [i], fichmail [i+1]);
            strcpy (fichmail [i+1], tampon);

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}



/* lit l'entte d'un message et affiche certaines lignes */

void aff_detail (char *nomfic)
{
    // buffers pour affichage ordonn des caractristiques du message
    char bufFrom [80], bufTo [80], bufCc [80], bufReply [80],
         bufSubject [80], bufContent [80];
    int  date_ok;  // indique si le champ date a t lu
    int i, j;      // simples compteurs


    // ouverture du fichier mail
    fmail = fopen (nomfic, "r");

    // arrt si erreur imprvue
    if (fmail == 0)
    {
        fprintf (stderr, "Impossible d'ouvrir le fichier %s\n", nomfic);
        return;
    }

    // initialisation
    bufFrom [0]    = '\0';
    bufTo [0]      = '\0';
    bufCc [0]      = '\0';
    bufReply [0]   = '\0';
    bufSubject [0] = '\0';
    bufContent [0] = '\0';
    date_ok = 0;

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_fmail ();

        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_fmail ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
        {
            // pas la peine de mmoriser la date si on l'affiche en premier
            printf ("Fichier mail %s\n", nomfic);
            puts (buf_lect);
            date_ok = 1;  // on sait que c'est un fichier mail
        }
        else if (start ("From"))
            memconvbuf (bufFrom);
        else if (start ("To"))
            memconvbuf (bufTo);
        else if (start ("Cc"))
            memconvbuf (bufCc);
        else if (start ("Reply-To"))
            memconvbuf (bufReply);
        else if (start ("Content-Type"))
        {
            membuf (bufContent);

            if (recup_ctype () == TextPlain)
                mem_charset ();
        }
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();
    }
    while (buf_lect [0] != '\0');  // lecture entte termine si ligne vide

    // si l'on n'a pas trouv de ligne Date: From: ou Subject:
    if (! date_ok && ! *bufFrom && ! *bufSubject)
        return;   // ce n'est pas un fichier mail

    // affichage des caractristiques mmorises
    if (bufFrom [0])
        puts (bufFrom);
    if (bufTo [0])
        puts (bufTo);
    if (bufCc [0])
        puts (bufCc);
    if (bufReply [0])
        puts (bufReply);
    if (bufSubject [0])
        puts (bufSubject);
    if (bufContent [0])
        puts (bufContent);

    // lecture et affichage du dbut du message
    i = 0;

    while (i++ < nblig_mes && lire_fmail ())
    {
        // on prend en compte les nouveaux champs Content ventuels
        if (start ("Content-Type") && recup_ctype () == TextPlain)
            mem_charset ();
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();

        // convertir la ligne lue et l'afficher
        majligne ();
        printf ("%s", buf_lect);
    }

    putchar ('\n');

    // librer le fichier mail
    fclose (fmail);
}



/* lit l'entte d'un message et affiche un rsum sur une ligne */

void aff_resume (char *nomfic)
{
    // buffers pour mmoriser les caractristiques principales du message
    char bufFromTo [80], bufSubject [80], bufDate [80];
    // les noms de mois dans les champs Date: des mails
    static char *mois [] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
    int affichedest; // indique si on doit afficher le champ destinataire
    int i, j;        // simples compteurs


    // ouverture du fichier mail
    fmail = fopen (nomfic, "r");

    // arrt si erreur imprvue
    if (fmail == 0)
    {
        fprintf (stderr, "Impossible d'ouvrir le fichier %s\n", nomfic);
        return;
    }

    // dterminer le champ  afficher entre expditeur et destinataire
    affichedest = (tolower (*nomfic) == 'e') || (tolower (*nomfic) == 's');

    // initialisation
    bufFromTo [0]  = '\0';
    bufSubject [0] = '\0';
    bufDate [0]    = '\0';

    // lecture et mmorisation des caractristiques du message
    do
    {
        // lire une ligne de l'entte du message
        lire_fmail ();

        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_fmail ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From") && ! affichedest)
            memconvbuf (bufFromTo);
        else if (start ("To") && affichedest)
            memconvbuf (bufFromTo);
    }
    while (buf_lect [0] != '\0');

    // si l'on n'a pas trouv de ligne Date: From: ou Subject:
    if (! *bufDate && ! *bufFromTo && ! *bufSubject)
        return;   // ce n'est pas un fichier mail

    // initialisation : on est en dbut de ligne
    i = 0;

    // affichage du nom de l'expditeur ou du destinataire du message
    if (*bufFromTo)
    {
        if (affichedest)
            j = 4;
        else
            j = 6;

        while (bufFromTo [j] == '"' || bufFromTo [j] == ' ')
            j++;

        putchar (bufFromTo [j++]);

        while (++i < sz_nom && bufFromTo [j] && bufFromTo [j] != '<'
                     && bufFromTo [j] != '"' && bufFromTo [j] != '=')
            putchar (bufFromTo [j++]);
    }

    // complt par des blancs
    while (i++ <= sz_nom)
        putchar (' ');

    // affichage du sujet du message
    if (*bufSubject)
    {
        j = 8;

        // on saute les blancs inutiles
        while (bufSubject [j] == ' ')
            j++;

        while (++i < sz_avantdate && bufSubject [j])
        {
            // si prsence d'une tabulation
            if (bufSubject [j] == '\t')
            {
                // afficher un blanc pour une meilleure mise en page
                putchar (' ');
                j++;
            }
            else
                // sinon affichage du caractre
                putchar (bufSubject [j++]);
        }
    }
    else
        i++;    // pour corriger l'alignement

    // complt par des blancs
    while (i++ <= sz_avantdate)
        putchar (' ');

    // affichage de la date et de l'heure du message
    if (*bufDate)
    {
        j = 6;

        // jour du mois
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j));

        while (bufDate [j] > ' ')
            j++;

        j++;

        // mois
        i = 0;

        while (i < 12 && (toupper (bufDate [j]) != mois [i][0]
                     || tolower (bufDate [j+1]) != mois [i][1]
                     || tolower (bufDate [j+2]) != mois [i][2]))
            i++;

        printf ("/%02d/", i + 1);

        // anne sur 2 chiffres
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j) % 100);

        while (bufDate [j] > ' ')
            j++;

        // heure, sans les secondes ni le fuseau horaire
        for (i = 0; i < 6; i++)
            putchar (bufDate [j++]);
    }

    // passage  la ligne
    putchar ('\n');

    // librer le fichier mail
    fclose (fmail);
}
