/*
    Fichier testchamp.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions permettant de rcuprer dans un fichier
    la liste des valeurs interdites dans un champ d'un mail puis de
    vrifier si un champ de mail contient l'une de ces valeurs.
*/

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "testchamp.h"



/* charge en mmoire le fichier contenant une liste de chaines de caractres */

int charge_valchamp (char *nomfic, char **liste, int max_sz_liste, char *nom_max_sz)
{
    char valchaine [80]; // une chaine de ce fichier
    char *nouvchaine;    // chaine retourne alloue dynamiquement
    FILE *fliste;        // descripteur du fichier des chaines  mmoriser
    int  sz_liste;       // nombre de chaines mmorises dans la liste
    int  i, j;           // compteurs


    // initialisation : pas encore de chaine mmorise
    sz_liste = 0;

    // accs au fichier contenant les chaines  mmoriser
    fliste = fopen (nomfic, "r");

    // si ce fichier existe
    if (fliste)
    {
        // rcuprer et mmoriser les diffrents chaines qu'il contient
        while (fgets (valchaine, sizeof (valchaine), fliste))
        {
            // supprimer le \n en fin de ligne
            valchaine [strlen (valchaine) - 1] = '\0';

            // allouer une variable pour mmoriser la chaine lue
            nouvchaine = (char *) malloc (strlen (valchaine) + 1);

            if (! nouvchaine)
            {
                fputs ("Problme d'allocation mmoire\n", stderr);
                exit (0);
            }

            // recopier la chaine
            strcpy (nouvchaine, valchaine);

            // mmoriser la chaine dans la liste ou message d'erreur
            if (sz_liste < max_sz_liste)
                liste [sz_liste ++] = nouvchaine;
            else
            {
                 fprintf (stderr,
                  "Trop de chaines  filtrer, le filtrage ne sera que partiel\n");
                 fprintf (stderr,
                   "Recompiler l'application avec %s plus grand\n", nom_max_sz);
                return;
            }
        }

        // termin avec ce fichier
        fclose (fliste);
    }
    else
    {
        // message d'erreur si fichier des chaines  mmoriser manquant
        fprintf (stderr, "Pas de fichier %s aucun filtrage possible\n", nomfic);
    }

    // retourner le nombre d'lments de la liste rcuprs
    return (sz_liste);
}



/* recherche si le champ du mail passe en paramtre
   contient l'une des chaines de la liste */

int trouve_valchamp (char *valchamp, char **liste, int sz_liste)
{
    char *chainetest;  // chaine de la liste  tester
    int  i, j, k;      // compteurs

    // on compare la chaine valchamp avec toutes les chaines de la liste
    for (i = 0; i < sz_liste; i++)
    {
        // permettra d'viter des calculs de dplacements dans tableau
        chainetest = liste [i];

        // initialisation
        k = 0;

        // vrifier si la chaine chainetest se trouve dans valchamp
        do
        {
            j = 0;

            while (valchamp [j+k] == chainetest [j])
                // termin si tous les caractres de chainetest trouvs
                if (chainetest [++j] == 0)
                    return 1;
        }
        // sinon recherche dans la suite de valchamp
        while (valchamp [k++]);
    }

    // aucune des chaines de la liste n'a t trouve dans valchamp
    return (0);
}
