/*
    Fichier vmaildir.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme affiche l'arborescence du rpertoire pass
    en paramtre, rpertoire courant par dfaut.

    Il est recommand de lancer cet outils depuis (ou en
    passant en paramtre) le rpertoire racine des fichiers
    mails tel qu'il est dfini dans le fichier de configuration
    xxxx.cfg

    Il est possible de slectionner l'un des rpertoires
    affichs pour consulter les fichiers mails qu'il contient
    et ventuellement rpondre, transfrer, ou dtruire le message.

    Pour ces oprations, cet outil appelle vmailsj
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "modepage.h"
#include "carspe.h"
#include "szchemin.h"


/*
   Par dfaut ce programme dtecte les noms
   de fichiers mails issus de libremail.

   Dcommenter la ligne qui suit pour accepter
   aussi ceux issus de microsoft internet mail.
*/
// #define compat_msoft


#define max_maildir   200  // nombre max de rpertoires de mails
#define lg_nomdir      32  // longueur max d'un nom de rpertoire

#define MAIL_PRESENT    1  // pour mmoriser prsence d'un mail dans rpertoire
#define NOUVEAUX_MAILS  3  // pour mmoriser mail non encore lu dans rpertoire


/* type de donnes */
typedef char nomdir [lg_nomdir];


/* prototypes */
void exploredir (char *repertoire);
int  fichier_mail (char *nomfic);
nomdir * maj_espace (nomdir * oldliste, int * taille);
void trinoms (nomdir *liste, int fin);
void navigation ();
void detectemail (int numrep);
void affligne (int numligne);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *maildir [max_maildir];  // liste des rpertoires de mails
char presmail [max_maildir];  // indique prsence de mails dans le rpertoire
int  numsortie = 0;           // position du rpertoire des mails  envoyer
int  numpoub   = 0;           // position du rpertoire des mails dtruits


/* programme principal */

int main (int nbarg, char *varg[])
{
    char nomdir [szchemin];
    char *ajout;


    // initialisation
    nb_lignes = 0;

    // rcupration du nom du rpertoire racine des mails
    if (--nbarg == 1)
    {
        // se positionner dans ce rpertoire
        if (chdir (varg [1]) < 0)
        {
            fprintf (stderr, "Rpertoire %s inexistant\n", varg [1]);
            return (0);
        }
    }

    // si la syntaxe d'appel de vmaildir est correcte
    if (nbarg < 2)
    {
        // rcuprer le chemin d'accs absolu de la racine des mails
        getcwd (nomdir, szchemin);

        // dterminer le nom du rpertoire d'envoi des mails
        ajout = nomdir + strlen (nomdir);
        strcpy (ajout, "/sortie");

        // et le mmoriser dans une variable d'environnement
        setenv ("mailenv", nomdir);

        // faire de mme avec le rpertoire poubelle des fichiers mail
        strcpy (ajout, "/poubelle");
        setenv ("mailpoub", nomdir);

        // mmoriser l'arborescence du rpertoire racine des mails
        exploredir (".");

        // afficher les rpertoire et naviguer dans la liste
        navigation ();
    }
    else
        fprintf (stderr, "Syntaxe : %s [rpertoire]\n", varg [0]);

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* exploration du rpertoire pass en paramtre */

void exploredir (char *repertoire)
{
    DIR *repert;
    struct dirent *entree;
    struct stat   descfic;
    char   nomfic [szchemin]; // chemin d'accs  un fichier
    nomdir *listedir;    // liste des sous rpertoires trouvs
    int    nbdir_alloc;  // nombre de sous repertoires allous
    int    nbdir;        // nombre de sous repertoires mmoriss
    int    i;            // simple compteur


    // accder au rpertoire
    repert = opendir (repertoire);

    // sortie sur erreur si l'on n'a pas pu
    if (repert == 0)
        return;

    // mmoriser le chemin d'accs au rpertoire
    if (nb_lignes == 0)
    {
        // cas particulier de rpertoire racine
        maildir [nb_lignes] = ".";
    }
    else if (nb_lignes < max_maildir)
    {
        // cas gnral : on ne mmorise pas le ./ des noms de sous rpertoires
        maildir [nb_lignes] = (char *) malloc (strlen (repertoire) - 1);

        if (maildir [nb_lignes])
            strcpy (maildir [nb_lignes], repertoire + 2);
        else
        {
            fputs ("Manque de place mmoire \n", stderr);
            exit (0);
        }

        // mmoriser la position des rpertoires sortie et poubelle
        if (strcmp (repertoire + 2, "sortie") == 0)
            numsortie = nb_lignes;
        else if (strcmp (repertoire + 2, "poubelle") == 0)
            numpoub = nb_lignes;
    }
    else
    {
        fputs ("Trop de sous rpertoires\n", stderr);
        fputs ("Recompiler avec une valeur de nb_lignes plus grande\n", stderr);
        exit (0);
    }

    // initialisation : pas encore de mail trouv dans ce rpertoire
    presmail [nb_lignes] = 0;

    // rcuprer le nombre de liens du rpertoire
    stat (repertoire, &descfic);

    // et allouer le bon nombre d'entres dans la liste des
    // sous rpertoires avec une scurit de 2 (correspondant
    //  . et ..) pour viter erreur de segmentation
    nbdir_alloc = descfic.st_nlink;
    listedir = (nomdir *) calloc (nbdir_alloc, lg_nomdir);

    if (! listedir)
    {
        fputs ("Manque de place mmoire \n", stderr);
        exit (0);
    }

    // sauter les rpertoire . et ..
    entree = readdir (repert);
    entree = readdir (repert);
    nbdir = 0;

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire
    while (entree)
    {
        // rcuprer les caractristiques du fichier courant
        sprintf (nomfic, "%s/%s", repertoire, entree->d_name);
        stat (nomfic, &descfic);

        // on ne mmorisera que les noms des rpertoires
        if ((descfic.st_mode & S_IFMT) == S_IFDIR)
        {
            // tronquer les noms  32 caractres
            if (strlen (entree->d_name) >= lg_nomdir)
                entree->d_name [lg_nomdir - 1] = '\0';

            // prcaution pour viter un dbordement de tableau
            if (nbdir >= nbdir_alloc)
                listedir = maj_espace (listedir, &nbdir_alloc);

            // mmoriser le nom du sous rpertoire
            strcpy (listedir [nbdir ++], entree->d_name);
        }
        // et la prsence de fichiers mail dj lus ou en attente de lecture
        else if (fichier_mail (entree->d_name))
        {
            presmail [nb_lignes] = presmail [nb_lignes] | MAIL_PRESENT;

            // le test de cette dernire lettre sert seulement 
            // la compatibilit avec microsoft internet mail
            if (entree->d_name [8] == '.' && tolower (entree->d_name [9]) == 'n')
                presmail [nb_lignes] = NOUVEAUX_MAILS;
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // librer le rpertoire
    closedir (repert);

    // mettre  jour le compteur
    nb_lignes ++;

    // tri alphabtique des noms de sous rpertoires
    trinoms (listedir, nbdir - 1);

    // exploration des sous rpertoires trouvs
    for (i = 0; i < nbdir; i++)
    {
        sprintf (nomfic, "%s/%s", repertoire, listedir [i]);
        exploredir (nomfic);
    }

    // libration de l'espace mmoire du tableau listedir
    free (listedir);
}



/* dtermine si un nom de fichier pass en paramtre
   correspond  un nom de fichier mail */

int fichier_mail (char *nomfic)
{
    char car; // pour viter de trop manipuler des tableaux
    int  i;   // compteur


    // test du premier caractre du nom de fichier
    car = tolower (nomfic [0]);

    // si le nom de fichier commence par l'une des bonnes lettres possibles
    if (car == 'r' || car == 'e'
#ifdef compat_msoft
                   || car == 's'
#endif
       )
        // on va examiner les caractres qui suivent
        i = 1;
    else
        // sinon ce n'est pas un fichier mail
        return 0;

    // test des caractres qui suivent
    while (i < 8)
    {
#ifdef compat_msoft
        car = tolower (nomfic [i]);
#else
        car = nomfic [i];
#endif

        // si le ime caractre est bon
        if (('0' <= car && car <= '9')
#ifdef compat_msoft
         || ('a' <= car && car <= 'z')
#endif
           )
            // examiner le caractre suivant
            i++;
        else
            // sinon ce n'est pas un fichier mail
            return 0;
    }

    // le si nom de fichier fait 8 caractres
    if (nomfic [8] == '\0')
        // il s'agit d'un nom de fichier mail
        return 1;

    // sinon si le 9me caractre est un . et qu'il y a 10 caractres en tout
    else if (nomfic [8] == '.' && nomfic [10] == '\0')
    {
        // examiner le dernier caractre
        car = tolower (nomfic [9]);

        if (car == 'n')
            return 1;    // fichier mail reu et non encore lu

#ifdef compat_msoft
        else if (car == 'r')
            return 1;   // fichier micro$oft internet mail dj lu
#endif
        else
            return 0;   // ce n'est pas un fichier mail
    }
    else
        // sinon ce n'est pas un fichier mail
        return 0;
}



/* augmentation de la taille d'une liste de sous rpertoires */

nomdir * maj_espace (nomdir * oldliste, int * taille)
{
    nomdir *newliste;    // nouveau tableau

    // allocation d'un tableau plus grand
    newliste = (nomdir *) calloc (*taille + 4, lg_nomdir);

    // controle
    if (! newliste)
    {
        fputs ("Manque de place mmoire \n", stderr);
        exit (0);
    }

    // copie de l'ancien tableau
    memcpy (newliste, oldliste, *taille * lg_nomdir);

    // libration de la mmoire
    free (oldliste);

    // retour de la nouvelle adrese et taille du tableau
    *taille = *taille + 4;
    return (newliste);
}



/* tri alphabtique des noms de sous rpertoires */

void trinoms (nomdir *liste, int fin)
{
    char tampon [lg_nomdir];  // chaine de caractres pour permutation
    int  i;   // indice dans la liste


    // initialisation
    i = 0;

    while (i < fin)
    {
        // on compare 2 lments conscutifs
        if (strcmp (liste [i], liste [i+1]) > 0)
        {
            // interversion de 2 noms
            strcpy (tampon, liste [i]);
            strcpy (liste [i], liste [i+1]);
            strcpy (liste [i+1], tampon);

            // ce qui peut supposer de comparer un nom dplac avec le prcdent
            if (i > 0)
                i--;
            else
                i++;
        }
        else
            i++;    // on progresse dans la liste
    }
}


/* affiche la liste des sous rpertoire et permet de
   la parcourir mme si elle tient sur plusieurs pages */

void navigation ()
{
    int car;     // caractre tap au clavier
    int erreurs; // nombre frappes caractres inconnus comme commandes


    // configurer la liaison clavier pour lecture directe avec timeout
    mode_raw ();

    // initialisation
    lignecran = 1;
    lignecour = 0;
    erreurs = 0;
    affpage ();

    do
    {
        // lire et traiter une touche du clavier
        car = leccar ();

        switch (car)
        {                  // dplacement dans la liste des rpertoires
            case MONTE   : monte (1);
                           erreurs = 0;
                           break;

            case DESCEND : descend (1);
                           erreurs = 0;
                           break;

            case PAGEUP  : monte (lignepage + lignecran - 2);
                           erreurs = 0;
                           break;

            case PAGEDOWN: descend (2 * lignepage - lignecran - 2);
                           erreurs = 0;
                           break;

            case HOME    : monte (lignecour);
                           erreurs = 0;
                           break;

            case FIN     : descend (nb_lignes - lignecour - 1);
                           erreurs = 0;
                           break;

            case 0       : // aide si trop d'erreurs ou  la demande
            case F1      : effpage ();
                           puts ("Touches utilisables :\n");
                           puts ("flches Pageup, Pagedown, Home et Fin");
                           puts ("pour se dplacer d'une ou plusieurs lignes");
                           puts ("v, l ou entre pour Voir (Lire)");
                           puts ("les mails du rpertoire");
                           puts ("n ou Inser pour en crer un Nouveau");
                           puts ("Control L pour rafficher la page");
                           puts ("Esc pour sortir de ce programme\n");
                           puts ("Appuyer sur une touche pour continuer");
                           leccar ();
                           // pas de break, on raffiche la page

            case CTRL    : affpage ();
                           erreurs = 0;
                           break;

            case 'l'     :
            case 'v'     : // ici, on regarde les descriptifs des mails
            case '\n'    : execom ("vmailsj", maildir [lignecour]);

                           // maj du mode d'affichage des rpertoires
                           detectemail (lignecour);
                           detectemail (numsortie);
                           detectemail (numpoub);

                           // rafficher la page d'cran
                           affpage ();
                           erreurs = 0;
                           break;

            case INSERT  : // cration d'un nouveau mail
            case 'n'     : execom ("cremail", " ");

                           // maj du mode affichage rpertoire mails  envoyer
                           detectemail (numsortie);

                           // rafficher la page d'cran
                           affpage ();
                           erreurs = 0;
            case ESC     : break;

                           // sortie aussi sur :q
            case ':'     : car = leccar ();

                           if (car == 'q')
                           {
                               car = ESC;
                               break;
                           }

            default      : putchar (7);  // bip

                           // si trop d'erreurs, afficher l'aide
                           if (++erreurs == 5)
                               ungetc (0, stdin);  // on ira sur l'aide
        }
    }
    while (car != ESC);

    // descendre en bas de page
    while (lignecran++ < lignepage && lignecour++ < nb_lignes)
        putchar ('\n');

    // retour  la configuration standard du clavier
    mode_normal ();
}



/* recherche de fichiers mail dans le rpertoire pass en paramtre */

void detectemail (int numrep)
{
    DIR *repert;
    struct dirent *entree;
    struct stat   descfic;
    char   nomfic [szchemin]; // chemin d'accs  un fichier


    // accder au rpertoire
    repert = opendir (maildir [numrep]);

    // sortie sur erreur si l'on n'a pas pu
    if (repert == 0)
        return;

    // initialisation : pas encore de mail trouv dans ce rpertoire
    presmail [numrep] = 0;

    // lire un nom de fichier
    entree = readdir (repert);

    // tant qu'il y a des noms de fichier  lire et
    // qu'on n'a pas trouv de mail en attente de lecture
    while (entree && presmail [numrep] != NOUVEAUX_MAILS)
    {
        // rcuprer les caractristiques du fichier courant
        sprintf (nomfic, "%s/%s", maildir [numrep], entree->d_name);
        stat (nomfic, &descfic);

        // si c'est un fichier
        if ((descfic.st_mode & S_IFMT) == S_IFREG)
        {
            // vrifier si son nom est celui d'un fichier mail dj lu
            if (strlen (entree->d_name) == 8)
                presmail [numrep] = presmail [numrep] | MAIL_PRESENT;

            // ou d'un fichiers mail en attente de lecture
            else if (strlen (entree->d_name) == 10 && entree->d_name [8] == '.')
            {
                // le test de cette dernire lettre sert seulement 
                // la compatibilit avec microsoft internet mail
                if (tolower (entree->d_name [9]) == 'n')
                    presmail [numrep] = NOUVEAUX_MAILS;
                else
                    presmail [numrep] = presmail [numrep] | MAIL_PRESENT;
            }
        }

        // et lire le nom du fichier suivant
        entree = readdir (repert);
    }

    // librer le rpertoire
    closedir (repert);
}



/* affiche le nom de rpertoire pass en
   paramtre sans dborder de la fentre */

void affligne (int numlig)
{
    int  i;
    char *carcourant;


    // positionnement au dbut du nom de rpertoire
    if (strcmp (maildir [numlig], ".") == 0)
        // cas particulier du rpertoire racine
        carcourant = "<racine>";
    else
        // cas gnral
        carcourant = maildir [numlig];

    // afficher en bleu fonc les rpertoires sans mail
    if (!presmail [numlig])
        sombre ();
    // et en surbrillance ceux qui ont des mails non lus
    else if (presmail [numlig] == NOUVEAUX_MAILS)
        clair ();

    // initialisation boucle
    i = 0;

    // on arrte l'affichage en fin de chaine ou en bord de fentre
    while (*carcourant  && i++ < colonpage)
    {
        putchar (*carcourant);
        carcourant ++;
    }

    // retour  l'affichage en luminosit normale
    lumnor ();
}
